import { useState, useEffect } from 'react';
import { FeatherIcon as GatherIcon } from 'lucide-react';
import ResourceSelectionModal from './modal/ResourceSelectionModal';
import CollectResourcesButton from './CollectResourcesButton';
import { Pet, Resources } from '../types/Pet';
import { updatePetAction, getPetGatheredResources, putPetCollectResources } from '../services/api/api';
import { PetAction } from '../types/PetUpdateActionRequest';
import { isGatheringAction, formatResourceName, getResourceFromAction } from '../utils/petUtils';

interface GatherResourcesButtonProps {
  pet: Pet;
  onGatherStart: () => void;
  onGatherComplete: (updatedPet: Pet) => void;
}

const resourceToActionMap: Record<string, PetAction> = {
  wisdom: 'GATHERING_WISDOM',
  gold: 'GATHERING_GOLD',
  food: 'GATHERING_FOOD'
};

export default function GatherResourcesButton({ pet, onGatherStart, onGatherComplete }: GatherResourcesButtonProps) {
  const [isModalOpen, setIsModalOpen] = useState(false);
  const [isGathering, setIsGathering] = useState(isGatheringAction(pet.petAction));
  const [gatheredResources, setGatheredResources] = useState<Resources>({ wisdom: 0, gold: 0, food: 0, junk: 0 });

  // Initialize gathering check if pet is already gathering
  useEffect(() => {
    if (isGatheringAction(pet.petAction)) {
      setIsGathering(true);
      const resources = getPetGatheredResources(pet.id).then(setGatheredResources);
      onGatherStart();
    }
  }, []);

  useEffect(() => {
    let interval: number;

    if (isGathering) {
      interval = setInterval(async () => {
        try {
          const resources = await getPetGatheredResources(pet.id);
          setGatheredResources(resources);
        } catch (error) {
          console.error('Failed to check gathered resources:', error);
        }
      }, 10000);
    }

    return () => {
      if (interval) {
        clearInterval(interval);
      }
    };
  }, [isGathering, pet.id]);

  useEffect(() => {
    setIsGathering(isGatheringAction(pet.petAction));
  }, [pet.petAction]);

  const handleGatherStart = async (resourceType: string) => {
    if (resourceType === 'stop') {
      try {
        await updatePetAction(pet.id, { petActionGather: 'IDLE' });
        setIsGathering(false);
        onGatherComplete(pet);
      } catch (error) {
        console.error('Failed to stop gathering:', error);
      }
      setIsModalOpen(false);
      return;
    }

    setIsGathering(true);
    onGatherStart();

    try {
      const petAction = resourceToActionMap[resourceType];
      const updatedPet = await updatePetAction(pet.id, { petActionGather: petAction });
      onGatherComplete(updatedPet);
    } catch (error) {
      console.error('Failed to gather resources:', error);
    } finally {
      setIsModalOpen(false);
    }
  };

  const handleCollect = async () => {
    try {
      const updatedPet = await putPetCollectResources(pet.id);
      setGatheredResources({ wisdom: 0, gold: 0, food: 0, junk: 0 });
      onGatherComplete(updatedPet);
    } catch (error) {
      console.error('Failed to collect resources:', error);
    }
  };

  const currentResource = getResourceFromAction(pet.petAction);

  return (
    <div>
      <button
        onClick={() => setIsModalOpen(true)}
        className={`flex items-center justify-center space-x-2
          bg-amber-900/30 hover:bg-amber-800/50
          border-2 border-amber-500/50 rounded-lg p-4
          transition-all duration-300 transform hover:scale-105 w-full`}
      >
        <GatherIcon className="w-6 h-6" />
        <span>
          {isGathering && currentResource
            ? `Gathering ${formatResourceName(currentResource)}...`
            : 'Gather Resources'}
        </span>
      </button>

      <CollectResourcesButton
        petId={pet.id}
        resources={gatheredResources}
        onCollect={handleCollect}
      />

      <ResourceSelectionModal
        isOpen={isModalOpen}
        onClose={() => setIsModalOpen(false)}
        onGather={handleGatherStart}
        pet={pet}
        isGathering={isGathering}
      />
    </div>
  );
}